@@ -1,104 +1,29 @@
-/**
- * A client-side 802.1x implementation 
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- *
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
- * The driver function for a Linux application layer EAPOL 
- * implementation
+ * Implementation for parsing configuration file, and storing needed data.
+ *
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * File: config.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: config.c,v 1.42 2004/08/19 02:28:06 chessing Exp $
- * $Date: 2004/08/19 02:28:06 $
+ * $Id: config.c,v 1.76 2005/08/21 18:12:12 chessing Exp $
+ * $Date: 2005/08/21 18:12:12 $
  * $Log: config.c,v $
- * Revision 1.42  2004/08/19 02:28:06  chessing
- * First piece of WPA patch.  (The patch is growing fast, and this commit is to save what is done so far.)
+ * Revision 1.76  2005/08/21 18:12:12  chessing
+ * Added the ability to reload our configuration using a HUP.  Cleaned up a bunch of memory leaks in the config parse code.
  *
- * Revision 1.41  2004/08/15 04:29:34  chessing
+ * Revision 1.75  2005/08/20 19:06:52  chessing
+ * Patch from Carsten Grohmann to fix a few things in xsup_get_state.c.  Also added the ability to define an empty network clause, that will set the card in to encryption disabled mode.  From there, anything short of changing the SSID will be ignored by Xsupplicant.
  *
- * Completed support for scanning and joining wireless networks.  If your interface isn't configured, we will scan to gather information about all known wireless networks.  If a network in the list has a configuration, we will join it.  (Including turning on needed encryption.)  There are still a few rough edges that need to be hammered out.  But the overall user experience should be improved.
+ * Revision 1.74  2005/08/18 03:19:04  chessing
+ * Added the ability to define an SSID with static WEP keys.  When we switch to a network that has this type of configuration we will set the keys, and stop the various association timers.
  *
- * Revision 1.40  2004/08/12 01:40:06  chessing
+ * Revision 1.73  2005/08/12 03:34:06  chessing
+ * Fix to the TLS implementation, should help avoid some of the weird 'block cipher pad' errors.  Also includes a partial implementation of the ability to use static WEP keys based on the SSID in use.
  *
- * Added "association" option to the global configuration file sections.  It can be set to either auto, or manual.  Setting "association" to auto means that XSupplicant will attempt to associate to a network with the lowest value in the "priority" setting.  The default for "association" is auto.  The other option added is "priority" which is a numeric value from 1..256.  A value of 1 is the highest priority, and will be the first network chosen.  (Assuming the network can be found in a scan.)  The default value for "priority" is 0, which means that network will be checked last.   --  Although the options are now in the config file, the actual functionality for auto association/priority is not yet complete.  (So, basically these options don't do anything yet. ;)
- *
- * Revision 1.39  2004/07/25 19:36:40  chessing
- * Fixed a few more logical/bitwise nots.  Added rtnetlink support for hot-plugging interfaces.  (Hot unplugging is coming soon. ;)
- *
- * Revision 1.38  2004/07/19 02:43:16  chessing
- *
- * Changed things to get rid of Linux specific pieces in the interface_data struct. Fixed up EAP-SIM and EAP-AKA to skip AVPs that we don't support.  (We print a mesage, and move on.)  Added the --enable-radiator-test option to configure EAP-AKA to use the test vectors included with Radiator's AKA module.  (To use this mode, no SIM card is required, but the PCSC library is still needed to build.  Also, some errors will be displayed.)
- *
- * Revision 1.37  2004/06/29 01:35:34  chessing
- *
- * Added credit to eapleap.c for Gilbert Goodwill (who gave me the clues I needed to figure out the keying piece for LEAP).  Added patch from Toby Collett to make PEAP work for broken implementations of IAS.  (New option ias_quirk has been added.)
- *
- * Revision 1.36  2004/06/22 20:14:22  chessing
- *
- * Fixed some compile problems where SIM/AKA code was trying to compile even when it wasn't supposed to.
- *
- * Revision 1.35  2004/06/21 05:19:12  chessing
- *
- * Added a few minor fixes to EAP-AKA support.  Added "allmulti" as a global configuration option.  (By default, allmulti is now on!)
- *
- * Revision 1.34  2004/06/15 03:35:19  chessing
- *
- * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
- *
- * Revision 1.33  2004/06/15 03:22:17  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.72  2005/08/09 01:39:13  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
@@ -107,6 +32,8 @@
 #include <strings.h>
 #include <string.h>
 #include <stdio.h>
+#include <sys/types.h>
+#include <grp.h>
 
 #include "xsup_debug.h"
 #include "xsup_err.h"
@@ -127,15 +54,18 @@
 #include "eap_types/sim/eapsim.h"
 #include "eap_types/aka/eapaka.h"
 
-
 struct config_data *config_info = NULL; // the configuration data structure
-int config_parse_debug = 1;
+struct config_network *current_network_data = NULL;  // Point to the config
+                                                     // data we currently are
+                                                     // using.
+
 extern FILE *yyin;
 extern int config_linenum;
 extern int yyparse(void);
 
 #define FREE_STRING(x) if (x != NULL) {free(x); x = NULL;}
 
+
 /****************************************
  *
  * Load all of the configuration information in to memory.  We should
@@ -147,7 +77,6 @@ int config_setup(char *path_to_config)
 {
   /* Make sure we got a parameter */
   if (path_to_config == NULL) {
-    printf("Doing Debug...\n");
     debug_printf(DEBUG_NORMAL, "Configuration file not given\n");
     debug_printf(DEBUG_NORMAL, "This should NEVER happen!\n");
     return XECONFIGFILEFAIL;
@@ -206,60 +135,41 @@ struct config_network *config_find_network(struct config_network *nethead,
   // Otherwise, look against the essid.
   cur = nethead;
 
-  if ((cur != NULL) && (cur->ssid != NULL))
-    {
-      while ((cur != NULL) && (strcmp(cur->ssid, matchname) != 0))
+  while (cur != NULL)
+  {
+    if ((cur->ssid == NULL) || (strcmp(cur->ssid,matchname) != 0)) 
 	{
 	  cur = cur->next;
+	} else {
+  	  break;
 	}
+  }
   
-      // Do we have a match on ssid?
-      if (cur != NULL)
-	{
-	  return cur;
-	}
-    }
+  // Do we have a match on ssid?
+  if (cur != NULL)
+  {
+	return cur;
+  }
 
   return NULL;
 }
 
 /****************************************
  *
- * Get the priority value based on a network name.
+ * Return our network data structure.
  *
  ****************************************/
-int config_get_priority(char *network_name)
+struct config_network *config_get_network_config()
 {
-  struct config_network *result;
-
-  if (config_info == NULL) 
-    debug_printf(DEBUG_NORMAL, "No valid configuration information to search!\n");
-
-  if (network_name == NULL)
-    debug_printf(DEBUG_NORMAL, "Invalid network name passed to %s.\n",
-		 __FUNCTION__);
-
-  result = config_find_network(config_info->networks, network_name);
-
-  if (result == NULL)
-    {
-      debug_printf(DEBUG_NORMAL, "Couldn't find configuration information for network %s!\n", network_name);
-      return 0;
-    }
-
-  debug_printf(DEBUG_CONFIG, "Returning Priority of %d\n", 
-	       result->priority);
-
-  return result->priority;
+  return current_network_data;
 }
 
 /****************************************
  *
- * Get configuration information out of memory, and populate the userdata
- * structure.
+ * Get configuration information out of memory, and 
  *
  ****************************************/
-struct config_network *config_build(char *network_name)
+char config_build(char *network_name)
 {
   struct config_network *result;
 
@@ -271,7 +181,11 @@ struct config_network *config_build(char *network_name)
       // to see if it matches any friendly names.
       result = config_find_network(config_info->networks, network_name);
 
-      if (result != NULL) return result;
+      if (result != NULL) 
+	{
+	  current_network_data = result;
+	  return TRUE;
+	}
 
       // This is not good.  We don't have any configuration information
       // for the requested network.  So, we need to return the default
@@ -281,49 +195,22 @@ struct config_network *config_build(char *network_name)
       result = config_find_network(config_info->networks, 
 				   config_info->globals->default_net);
 
-      if (result != NULL) return result;
+      if (result != NULL) 
+	{
+	  current_network_data = result;
+	  return TRUE;
+	}
 
       // Uh oh..  We didn't find *anything*.
       debug_printf(DEBUG_NORMAL, "ERROR : No valid network profile could be located!  (Even tried default.)\n");
-
+      debug_printf(DEBUG_NORMAL, "Abnormal program termination!!!!\n");
+      exit(1);
     } else {
       debug_printf(DEBUG_CONFIG, "config_info == NULL!  No config to update!\n");
     }
-  return NULL;
+  return FALSE;
 }
 
-/*********************************************
- *
- * Determine if the interface given is in the list of interfaces.
- *
- *********************************************/
-int config_int_in_list(char *intName, struct config_string_list *strList)
-{
-  struct config_string_list *cur;
-
-  if ((!intName) || (!strList))
-    {
-      debug_printf(DEBUG_NORMAL, "Invalid data passed in to int_in_list()!\n");
-      return XEMALLOC;
-    }
-
-  // Our check is case sensative!
-  cur = strList;
-
-  while ((cur != NULL) && (strcmp(intName, cur->name) != 0))
-    {
-      cur = cur->next;
-    }
-
-  if (cur == NULL)
-    {
-      return FALSE;    // The interface isn't in our list.
-    } 
-
-  return TRUE;
-}
-
-
 /************************************
  *
  * Set statemachine/config related variables for this interface.
@@ -361,84 +248,146 @@ int config_set_globals(struct interface_data *myint)
 
 /************************************
  *
- * Return a list of allowed interfaces.
+ * Return the GID of the group defined by ipc_group.
  *
  ************************************/
-struct config_string_list *config_allowed_interfaces()
+int config_get_ipc_gid()
 {
-  return config_info->globals->allow_interfaces;
+  struct group *grdata = NULL;
+  int result = -1;
+
+  if (config_info->globals->ipc_group_name != NULL)
+    {
+      grdata = getgrnam(config_info->globals->ipc_group_name);
+      if (grdata != NULL)
+	{
+	  result = grdata->gr_gid;
+	  debug_printf(DEBUG_CONFIG, "IPC Group Name %s, number %d!\n",
+		       grdata->gr_name, result);
+	}
+    }
+
+  return result;
 }
 
 /************************************
  *
- * Return a list of denied interfaces.
+ * Return the way that we want to determine the destination for packets.
  *
  ************************************/
-struct config_string_list *config_denied_interfaces()
+char config_get_destination()
 {
-  return config_info->globals->deny_interfaces;
+  if ((config_info == NULL) || (config_info->globals == NULL))
+    {
+      debug_printf(DEBUG_CONFIG, "No configuration information available!  "
+		   "Returning destination of AUTO.\n");
+      return DEST_AUTO;
+    }
+
+  return config_info->globals->destination;
 }
 
 /************************************
  *
- * Return the startup command we want to use.  The caller should *NOT* free
- * the resulting variable!
+ * Get stale key timer.
  *
  ************************************/
-char *config_get_startup_cmd()
+int config_get_stale_key_timeout()
 {
   if ((config_info == NULL) || (config_info->globals == NULL))
     {
-      debug_printf(DEBUG_CONFIG, "(startup_cmd) No configuration information available!\n");
-      return NULL;
+      debug_printf(DEBUG_CONFIG, "Invalid configuration information!  Stale"
+		   " key timer will be set to the default of %d seconds!\n",
+		   STALE_KEY_WARN_TIMEOUT);
+      return STALE_KEY_WARN_TIMEOUT;
     }
-
-  return config_info->globals->startup_command;
+  return config_info->globals->stale_key_timeout;
 }
 
-char *config_get_first_auth_cmd()
+/************************************
+ *
+ * Return the default interface that was in the configuration file.
+ *
+ ************************************/
+char *config_get_default_int()
 {
   if ((config_info == NULL) || (config_info->globals == NULL))
     {
-      debug_printf(DEBUG_CONFIG, "(first_auth_cmd) No configuration information available!\n");
+      debug_printf(DEBUG_CONFIG, "No default interface set in the "
+		   "configuration file!\n");
       return NULL;
     }
-  return config_info->globals->first_auth_command;
+
+  return config_info->globals->default_int;
 }
 
-char *config_get_reauth_cmd()
+/************************************
+ *
+ * Return the log_facility value.
+ *
+ ************************************/
+char *config_get_log_facility()
 {
   if ((config_info == NULL) || (config_info->globals == NULL))
     {
-      debug_printf(DEBUG_CONFIG, "(reauth_cmd) No configuration information available!\n");
+      debug_printf(DEBUG_CONFIG, "No log facility defined!\n");
       return NULL;
     }
-  return config_info->globals->reauth_command;
+
+  return config_info->globals->log_facility;
 }
 
+/***********************************
+ *
+ * Return if we should display friendly warnings or not.
+ *
+ ***********************************/
+char config_get_display_friendly_warnings()
+{
+  if ((config_info == NULL) || (config_info->globals == NULL))
+    {
+      debug_printf(DEBUG_CONFIG, "No configuration information available! "
+		   "We will display all friendly warnings.\n");
+      return TRUE;
+    }
+
+  if (TEST_FLAG(config_info->globals->flags, 
+		CONFIG_GLOBALS_NO_FRIENDLY_WARNINGS))
+    {
+      return FALSE;  // Don't display friendly warnings.
+    } else {
+      return TRUE;
+    }
+
+  return TRUE;
+}
+
+/***********************************
+ *
+ * Return if we should set allmulti mode, or not.
+ *
+ ***********************************/
 char config_get_allmulti()
 {
   if ((config_info == NULL) || (config_info->globals == NULL))
     {
-      debug_printf(DEBUG_CONFIG, "No configuration information available!  ALLMULTI will return default value!\n");
+      debug_printf(DEBUG_CONFIG, "No configuration information available!  "
+		   "ALLMULTI will return default value!\n");
       return 1;
     }
-  return config_info->globals->allmulti;
-}
 
-char config_get_association()
-{
-  if ((config_info == NULL) || (config_info->globals == NULL))
+  if (TEST_FLAG(config_info->globals->flags, CONFIG_GLOBALS_ALLMULTI))
     {
-      debug_printf(DEBUG_CONFIG, "No configuration information available!  ASSOCIATION will return default value!\n");
-      return 0;
+      return TRUE;
+    } else {
+      return FALSE;
     }
-  return config_info->globals->association;
 }
 
-/**********************************************************************************
- * NOTE: Do *NOT* debug_printf() in this function or you'll cause a recursive loop.
- **********************************************************************************/
+/*****************************************************************************
+ * NOTE: Do *NOT* debug_printf() in this function or you'll cause a recursive 
+ * loop.
+ *****************************************************************************/
 char *config_get_logfile()
 {
   if ((config_info == NULL) || (config_info->globals == NULL))
@@ -523,6 +472,55 @@ int config_allows_network(struct config_data *conf, char *netname)
 // not call these from outside config code
 //**********************************************
 
+  /*********************/
+ /* CONFIG_STATIC_WEP */
+/*********************/
+
+void delete_config_static_wep(struct config_static_wep **tmp_static_wep)
+{
+  int i;
+
+  if (*tmp_static_wep == NULL)
+    return;
+
+  for (i=1;i<5;i++)
+    {
+      FREE_STRING((*tmp_static_wep)->key[i]);
+    }
+
+  free(*tmp_static_wep);
+  *tmp_static_wep = NULL;
+}
+
+void initialize_config_static_wep(struct config_static_wep **tmp_static_wep)
+{
+  if (*tmp_static_wep != NULL) {
+    delete_config_static_wep(tmp_static_wep);
+  }
+  *tmp_static_wep = 
+    (struct config_static_wep *)malloc(sizeof(struct config_static_wep));
+  if (*tmp_static_wep)
+    memset(*tmp_static_wep, 0, sizeof(struct config_static_wep));
+}
+
+void dump_config_static_wep(struct config_static_wep *tmp_static_wep)
+{
+  int i;
+
+  if (!tmp_static_wep)
+    return;
+  debug_printf(DEBUG_NORMAL, "\t-------------static-wep-------------\n");
+    debug_printf(DEBUG_NORMAL, "\t TX Key       : %d\n", tmp_static_wep->tx_key);
+
+    for (i=1;i<5;i++)
+      {
+	if (tmp_static_wep->key[i])
+	  debug_printf(DEBUG_NORMAL, "\t Static Key %d : %s\n",
+		       tmp_static_wep->key[i]);
+      }
+  debug_printf(DEBUG_NORMAL, "\t------------------------------------\n");
+}
+
   /*******************/
  /* CONFIG_WPA_PSK  */
 /*******************/
@@ -578,6 +576,9 @@ void delete_config_eap_tls(struct config_eap_tls **tmp_tls)
   FREE_STRING((*tmp_tls)->user_key);
   FREE_STRING((*tmp_tls)->user_key_pass);
   FREE_STRING((*tmp_tls)->random_file);
+  FREE_STRING((*tmp_tls)->sc.engine_id);
+  FREE_STRING((*tmp_tls)->sc.opensc_so_path);
+  FREE_STRING((*tmp_tls)->sc.key_id);
   
   free (*tmp_tls);
   *tmp_tls = NULL;
@@ -592,7 +593,10 @@ void initialize_config_eap_tls(struct config_eap_tls **tmp_tls)
   *tmp_tls = 
     (struct config_eap_tls *)malloc(sizeof(struct config_eap_tls));
   if (*tmp_tls)
-    memset(*tmp_tls, 0, sizeof(struct config_eap_tls));
+    {
+      memset(*tmp_tls, 0, sizeof(struct config_eap_tls));
+      bzero(&(*tmp_tls)->sc, sizeof((*tmp_tls)->sc));
+    }
 }
 
 void dump_config_eap_tls(struct config_eap_tls *tls)
@@ -622,6 +626,12 @@ void dump_config_eap_tls(struct config_eap_tls *tls)
       debug_printf_nl(DEBUG_NORMAL, "NO\n");
       break;
     }
+  debug_printf(DEBUG_NORMAL, "\tUsing Smartcard:\n");
+  debug_printf(DEBUG_NORMAL, "\t\tEngine        : \"%s\"\n", tls->sc.engine_id);
+  debug_printf(DEBUG_NORMAL, "\t\tOpensc SO_PATH: \"%s\"\n", tls->sc.opensc_so_path);
+  debug_printf(DEBUG_NORMAL, "\t\tKey ID        : \"%s\"\n", tls->sc.key_id);
+  //debug_printf(DEBUG_NORMAL, "\t\tCertificate ID: \"%s\"", tls.sc->cert_id);
+  //debug_printf(DEBUG_NORMAL, "\t\tRoot Cert ID  : \"%s\"", tls.sc->root_id); 
   debug_printf(DEBUG_NORMAL, "\t------------------------------------\n");
 }
 
@@ -878,6 +888,8 @@ int config_ttls_phase2_contains_phase2(struct config_ttls_phase2 *phase2,
 
 void delete_config_ttls_phase2 (struct config_ttls_phase2 **phase2)
 {
+  struct config_ttls_phase2 *temp;
+
   if (*phase2 == NULL)
     return;
   switch ((*phase2)->phase2_type) {
@@ -898,8 +910,11 @@ void delete_config_ttls_phase2 (struct config_ttls_phase2 **phase2)
 		 " type.\nNotify developers. Type: 0x%x\n", 
 		 (*phase2)->phase2_type);
   }
-  if ((*phase2)->next)
-    delete_config_ttls_phase2(&(*phase2)->next);
+  temp = (*phase2)->next;
+  free((*phase2));
+
+  if (temp)
+    delete_config_ttls_phase2(&temp);
 }
 
 void dump_config_ttls_phase2(struct config_ttls_phase2 *phase2) {
@@ -957,6 +972,7 @@ void initialize_config_eap_ttls(struct config_eap_ttls **tmp_ttls)
   if (*tmp_ttls == NULL)
     return;
   memset(*tmp_ttls, 0, sizeof(struct config_eap_ttls));
+
   (*tmp_ttls)->phase2_type = TTLS_PHASE2_UNDEFINED;
 }
 
@@ -1146,7 +1162,9 @@ void initialize_config_eap_peap(struct config_eap_peap **tmp_peap)
   *tmp_peap = 
     (struct config_eap_peap *)malloc(sizeof(struct config_eap_peap));  
   if (*tmp_peap)
-    memset(*tmp_peap, 0, sizeof(struct config_eap_peap));
+    {
+      memset(*tmp_peap, 0, sizeof(struct config_eap_peap));
+    }
 }
 
 void dump_config_eap_peap(struct config_eap_peap *peap)
@@ -1181,8 +1199,18 @@ void dump_config_eap_peap(struct config_eap_peap *peap)
       debug_printf_nl(DEBUG_NORMAL, "NO\n");
       break;
     }
-   debug_printf(DEBUG_NORMAL, "\t  PEAP IAS_QUIRK: \"%s\"\n", 
-		 peap->ias_quirk ? "Yes" : "No");
+  debug_printf(DEBUG_NORMAL, "\t  Broken PEAPv1 Keying : ");
+  switch (peap->proper_peapv1)
+    {
+    case 0:
+      debug_printf_nl(DEBUG_NORMAL, "NO\n");
+      break;
+    case 1:
+      debug_printf_nl(DEBUG_NORMAL, "YES\n");
+      break;
+    }
+  debug_printf(DEBUG_NORMAL, "\t  PEAP IAS_QUIRK: \"%s\"\n", 
+	       peap->ias_quirk ? "Yes" : "No");
     
   if (TEST_FLAG(peap->flags, CONFIG_PEAP_ALLOW_MSCV2))
     debug_printf(DEBUG_NORMAL,"\t   Allow Phase 2 Type: MSCHAPv2\n");
@@ -1335,9 +1363,14 @@ void add_config_eap_method(struct config_eap_method **method,
 
 void delete_config_eap_method(struct config_eap_method **method)
 {
+  struct config_eap_method *temp;
+
   if (*method == NULL)
     return;
   switch ((*method)->method_num) {
+  case STATIC_WEP_METHOD:
+    delete_config_static_wep((struct config_static_wep **)&((*method)->method_data));
+    break;
   case WPA_PSK:
     delete_config_wpa_psk((struct config_wpa_psk **)&((*method)->method_data));
     break;
@@ -1375,8 +1408,12 @@ void delete_config_eap_method(struct config_eap_method **method)
 		 " type.\nNotify developers. Type: 0x%x\n", 
 		 (*method)->method_num);
   }
-  if ((*method)->next)
-    delete_config_eap_method(&(*method)->next);
+
+  temp = (*method)->next;
+  free((*method));
+
+  if (temp)
+    delete_config_eap_method(&temp);
   
 }
 
@@ -1385,6 +1422,9 @@ void dump_config_eap_method(struct config_eap_method *method, int dumplevel)
   if (method == NULL)
     return;
   switch ((method)->method_num) {
+  case STATIC_WEP_METHOD:
+    dump_config_static_wep((struct config_static_wep *)((method)->method_data));
+    break;
   case WPA_PSK:
     dump_config_wpa_psk((struct config_wpa_psk *)((method)->method_data));
     break;
@@ -1503,6 +1543,36 @@ void config_network_add_net(struct config_network **list,
   (*current) = toadd;
 }
 
+void dump_config_network_wpa(u_char crypt)
+{
+  switch (crypt)
+    {
+    case CRYPT_WEP40:
+      debug_printf(DEBUG_NORMAL, "WEP40\n");
+      return;
+
+    case CRYPT_TKIP:
+      debug_printf(DEBUG_NORMAL, "TKIP\n");
+      return;
+
+    case CRYPT_WRAP:
+      debug_printf(DEBUG_NORMAL, "WRAP\n");
+      return;
+
+    case CRYPT_CCMP:
+      debug_printf(DEBUG_NORMAL, "CCMP\n");
+      return;
+
+    case CRYPT_WEP104:
+      debug_printf(DEBUG_NORMAL, "WEP104\n");
+      return;
+    
+    default:
+      debug_printf(DEBUG_NORMAL, "NONE\n");
+      return;
+    }
+}
+
 void dump_config_network(struct config_network *net)
 {
   if (!net)
@@ -1516,6 +1586,21 @@ void dump_config_network(struct config_network *net)
   else
     debug_printf(DEBUG_NORMAL, "  Type: WIRELESS\n");
 
+  switch (net->force_eapol_ver)
+    {
+    case 1:
+      debug_printf(DEBUG_NORMAL, "  Forced EAPOL Version : 1\n");
+      break;
+      
+    case 2:
+      debug_printf(DEBUG_NORMAL, "  Forced EAPOL Version : 2\n");
+      break;
+
+    default:
+      debug_printf(DEBUG_NORMAL, "  Forced EAPOL Version : Default\n");
+      break;
+    }
+
   if (net->wireless_ctrl == CTL_UNSET)
     debug_printf(DEBUG_NORMAL, "  Wireless Control: UNSET\n");
   else if (net->wireless_ctrl == CTL_YES)
@@ -1523,29 +1608,11 @@ void dump_config_network(struct config_network *net)
   else
     debug_printf(DEBUG_NORMAL, "  Wireless Control: NO\n");
 
-  debug_printf(DEBUG_NORMAL, "  Crypt Method: ");
-  switch (net->crypt_type)
-    {
-    case CRYPT_ANY:
-      debug_printf(DEBUG_NORMAL, "ANY\n");
-      break;
-
-    case CRYPT_WEP:
-      debug_printf(DEBUG_NORMAL, "WEP\n");
-      break;
-
-    case CRYPT_WPA_TKIP:
-      debug_printf(DEBUG_NORMAL, "WPA-TKIP\n");
-      break;
-      
-    case CRYPT_WPA_CCMP:
-      debug_printf(DEBUG_NORMAL, "WPA-CCMP\n");
-      break;
-
-    case CRYPT_WPA_ANY:
-      debug_printf(DEBUG_NORMAL, "WPA-ANY\n");
-      break;
-    }
+  debug_printf(DEBUG_NORMAL, "  WPA Group Crypt : ");
+  dump_config_network_wpa(net->wpa_group_crypt);
+  
+  debug_printf(DEBUG_NORMAL, "  WPA Pairwise Crypt : ");
+  dump_config_network_wpa(net->wpa_pairwise_crypt);
 
   if (TEST_FLAG(net->flags, CONFIG_NET_ALLOW_WPA_PSK))
     debug_printf(DEBUG_NORMAL, "  Allow Type: WPA-PSK\n");
@@ -1595,7 +1662,6 @@ void dump_config_network(struct config_network *net)
 
   debug_printf(DEBUG_NORMAL, "  SSID: \"%s\"\n", net->ssid);
   debug_printf(DEBUG_NORMAL, "  Identity: \"%s\"\n", net->identity);
-  debug_printf(DEBUG_NORMAL, "  Priority: %d\n", net->priority);
 
   if (TEST_FLAG(net->flags, CONFIG_NET_DEST_MAC))
     debug_printf(DEBUG_NORMAL, "  DEST MAC: %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n",
@@ -1686,15 +1752,10 @@ void delete_config_globals(struct config_globals **tmp_globals)
   if ((*tmp_globals)->allowed_nets)
     delete_config_string_list(&(*tmp_globals)->allowed_nets);
 
-  FREE_STRING((*tmp_globals)->startup_command);
-  FREE_STRING((*tmp_globals)->first_auth_command);
-  FREE_STRING((*tmp_globals)->reauth_command);
   FREE_STRING((*tmp_globals)->logfile);
-  
-  if ((*tmp_globals)->allow_interfaces)
-    delete_config_string_list(&(*tmp_globals)->allow_interfaces);
-  if ((*tmp_globals)->deny_interfaces)
-    delete_config_string_list(&(*tmp_globals)->deny_interfaces);
+  FREE_STRING((*tmp_globals)->ipc_group_name);
+  FREE_STRING((*tmp_globals)->log_facility);
+  FREE_STRING((*tmp_globals)->default_int);
   
   free (*tmp_globals);
   *tmp_globals = NULL;
@@ -1711,7 +1772,9 @@ void initialize_config_globals(struct config_globals **tmp_globals)
     memset(*tmp_globals, 0, sizeof(struct config_globals));
 
   // We want to set allmulti to have a default of 1.
-  (*tmp_globals)->allmulti = 1;
+  SET_FLAG((*tmp_globals)->flags, CONFIG_GLOBALS_ALLMULTI);
+
+  (*tmp_globals)->stale_key_timeout = STALE_KEY_WARN_TIMEOUT;
 }
 
 void dump_config_globals(struct config_globals *globals)
@@ -1726,19 +1789,22 @@ void dump_config_globals(struct config_globals *globals)
   else 
     debug_printf(DEBUG_NORMAL, "Allowed Nets: ALL\n");
 
-  if (globals->startup_command)
-    debug_printf(DEBUG_NORMAL,
-		 "Startup Command: '%s'\n", globals->startup_command);
-  if (globals->first_auth_command)
-    debug_printf(DEBUG_NORMAL,
-		 "First Auth Command: '%s'\n", globals->first_auth_command);
-  if (globals->reauth_command)
-    debug_printf(DEBUG_NORMAL,
-		 "Re-Auth Command: '%s'\n", globals->reauth_command);
   if (globals->logfile)
     debug_printf(DEBUG_NORMAL,
 		 "Logfile: '%s'\n", globals->logfile);
 
+  if (globals->log_facility)
+    debug_printf(DEBUG_NORMAL,
+		 "Log Facility: '%s'\n", globals->log_facility);
+
+  if (globals->ipc_group_name)
+    debug_printf(DEBUG_NORMAL, "IPC Group Name : '%s' \n",
+		 globals->ipc_group_name);
+
+  if (globals->default_int)
+    debug_printf(DEBUG_NORMAL, "Default Interface : '%s' \n",
+		 globals->default_int);
+
   if (TEST_FLAG(globals->flags, CONFIG_GLOBALS_AUTH_PER))
     debug_printf(DEBUG_NORMAL, "Auth Period: %d\n", globals->auth_period);
   if (TEST_FLAG(globals->flags, CONFIG_GLOBALS_HELD_PER))
@@ -1746,25 +1812,37 @@ void dump_config_globals(struct config_globals *globals)
   if (TEST_FLAG(globals->flags, CONFIG_GLOBALS_MAX_STARTS))
     debug_printf(DEBUG_NORMAL,"Max Starts: %d\n", globals->max_starts);
 
-  if (globals->allmulti == 1)
+  if (TEST_FLAG(globals->flags, CONFIG_GLOBALS_NO_FRIENDLY_WARNINGS))
+    {
+      debug_printf(DEBUG_NORMAL, "Friendly Warnings : Yes\n");
+    } else {
+      debug_printf(DEBUG_NORMAL, "Friendly Warnings : No\n");
+    }
+
+  if (TEST_FLAG(globals->flags, CONFIG_GLOBALS_ALLMULTI))
     debug_printf(DEBUG_NORMAL, "ALLMULTI is enabled!\n");
   else
     debug_printf(DEBUG_NORMAL, "ALLMULTI is disabled!\n");
 
-  if (globals->association == 0)
-    debug_printf(DEBUG_NORMAL, "ASSOCIATION is set to AUTO!\n");
-  else
-    debug_printf(DEBUG_NORMAL, "ASSOCIATION is set to MANUAL!\n");
+  debug_printf(DEBUG_NORMAL, "Destination: ");
+  switch (globals->destination)
+    {
+    case DEST_AUTO:
+      debug_printf(DEBUG_NORMAL, "Auto\n");
+      break;
 
-  if (globals->allow_interfaces)
-    dump_config_string_list(globals->allow_interfaces, "Interface Allowed");
-  else 
-    debug_printf(DEBUG_NORMAL, "Allowed Interfaces: ALL\n");
+    case DEST_BSSID:
+      debug_printf(DEBUG_NORMAL, "BSSID\n");
+      break;
 
-  if (globals->deny_interfaces)
-    dump_config_string_list(globals->deny_interfaces, "Interface Denied");
-  else 
-    debug_printf(DEBUG_NORMAL, "Denied Interfaces: NONE\n");
+    case DEST_MULTICAST:
+      debug_printf(DEBUG_NORMAL, "Multicast\n");
+      break;
+
+    case DEST_SOURCE:
+      debug_printf(DEBUG_NORMAL, "Source\n");
+      break;
+    }
 }
 
   /*******************/
@@ -1794,7 +1872,12 @@ void initialize_config_data(struct config_data **tmp_data)
   *tmp_data = 
     (struct config_data *)malloc(sizeof(struct config_data));  
   if (*tmp_data)
-    memset(*tmp_data, 0, sizeof(struct config_data));
+    {
+      memset(*tmp_data, 0, sizeof(struct config_data));
+    } else {
+      debug_printf(DEBUG_NORMAL, "Couldn't allocate memory for configuration"
+		   " data!\n");
+    }
 }
 
 void dump_config_data(struct config_data *data)
