@@ -1,61 +1,6 @@
-/**
- * A client-side 802.1x implementation 
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- *
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
- * The driver function for a Linux application layer EAPOL 
- * implementation
- * File: config.c
+ *
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
  *
  * Authors: Chris.Hessing@utah.edu
  *
@@ -70,10 +15,28 @@
 #define UNSET_FLAG(var,flag)  (var &= ~flag)
 #define TEST_FLAG(var,flag)   (var & flag)
 
+#define DEST_AUTO      0
+#define DEST_BSSID     1
+#define DEST_MULTICAST 2
+#define DEST_SOURCE    3
+
 // if you change this, update the printing and parsing functions
 // accordingly
 #define CONFIG_MAC_LEN 6
 
+// If a WEP key sits around for more than the timeout below, we should barf
+// out a warning letting the user know that their data isn't as secure as
+// it could be.
+#define STALE_KEY_WARN_TIMEOUT 600   // 10 minutes.
+
+// The amount of time we should wait before we assume an authentication will
+// not be able to succeed.
+#define AUTHENTICATION_TIMEOUT  30   // 30 seconds.
+
+// Bogus invalid EAP method # that indicates that we are talking about
+// static WEP.
+#define STATIC_WEP_METHOD   -2
+
 typedef enum {RES_UNSET, RES_YES, RES_NO} sess_res;
 
 /*** DEVELOPER CHECKLIST ****/
@@ -84,6 +47,22 @@ typedef enum {RES_UNSET, RES_YES, RES_NO} sess_res;
     4. modify the grammar to account for the new fields (config_grammar.y)
     5. modify the lexicon for the new tokens (config_lexicon.l)
 */
+
+struct smartcard
+{
+  char *engine_id;
+  char *opensc_so_path;
+  char *key_id;
+  //char *cert_id;
+  //char *rootcert_id;
+};
+
+struct config_static_wep
+{
+  u_char *key[5];        // Index 0 should NEVER be used!
+  u_char tx_key;
+};
+
 struct config_eap_tls 
 {
   char * user_cert;
@@ -95,6 +74,7 @@ struct config_eap_tls
   sess_res session_resume;
   int chunk_size;
   char * random_file;
+  struct smartcard sc;
 
 };
 
@@ -146,6 +126,8 @@ struct config_eap_otp
   char *password;
 };
 
+// The items in this structure need to match those in config_eap_tls up to 
+// the random_file item, or else things may have problems.
 struct config_eap_ttls
 {
   char * user_cert;
@@ -154,11 +136,12 @@ struct config_eap_ttls
   char *crl_dir;
   char * user_key;
   char * user_key_pass;
-  char *random_file;
-  char *cncheck;
   sess_res session_resume;
-  int  cnexact;
   int  chunk_size;
+  char *random_file;
+
+  char *cncheck;
+  int  cnexact;
 
   ttls_phase2_type phase2_type; //the type to actually do
   struct config_ttls_phase2 *phase2; // all types with info defined
@@ -177,20 +160,26 @@ struct config_eap_mschapv2
   char *password;
 };
 
+// The items in this structure need to match those in config_eap_tls up to 
+// the random_file item, or else things may have problems.
 struct config_eap_peap
 {
-  char *identity; // phase2 identity
   char * user_cert;
   char *root_cert;
   char *root_dir;
   char *crl_dir;
   char * user_key;
   char * user_key_pass;
-  char *random_file;
-  char *cncheck;
   sess_res session_resume;
+  int  chunk_size;
+  char *random_file;
+
+  char *cncheck;
+  char proper_peapv1;
   int cnexact;
-  int chunk_size;
+
+  char *identity; // phase2 identity
+
 #define CONFIG_PEAP_ALLOW_MSCV2   0x00000001
 #define CONFIG_PEAP_ALLOW_MD5     0x00000002
 #define CONFIG_PEAP_ALLOW_SIM     0x00000004
@@ -275,22 +264,23 @@ struct config_network
 
   char *name;
   int flags;
-  int priority;
   enum {UNSET, WIRED, WIRELESS} type;
   char *ssid;
   char *identity;
+  char force_eapol_ver;
   enum {CTL_UNSET, CTL_YES, CTL_NO}  wireless_ctrl;
 
   u_char  dest_mac[CONFIG_MAC_LEN];
 
   // Different types of encryption that are allowed.
-#define CRYPT_ANY         0
-#define CRYPT_WEP         1
-#define CRYPT_WPA_TKIP    2
-#define CRYPT_WPA_CCMP    3
-#define CRYPT_WPA_ANY     4
+#define CRYPT_WEP40       1
+#define CRYPT_TKIP        2
+#define CRYPT_WRAP        3
+#define CRYPT_CCMP        4
+#define CRYPT_WEP104      5
 
-  char crypt_type;
+  u_char wpa_group_crypt;
+  u_char wpa_pairwise_crypt;
 
   // EAP Methods that can be in the config file
   struct config_eap_method *methods; 
@@ -313,23 +303,22 @@ struct config_globals
 {
   char *default_net;
   struct config_string_list *allowed_nets;  
-  char *startup_command;
-  char *first_auth_command;
-  char *reauth_command;
   char *logfile;
+  char *default_int;
+  char *log_facility;
+  char *ipc_group_name;
   // the following indicate the values below are set and should be used
-#define CONFIG_GLOBALS_AUTH_PER    0x00000001
-#define CONFIG_GLOBALS_HELD_PER    0x00000002
-#define CONFIG_GLOBALS_MAX_STARTS  0x00000004
+#define CONFIG_GLOBALS_AUTH_PER             0x00000001
+#define CONFIG_GLOBALS_HELD_PER             0x00000002
+#define CONFIG_GLOBALS_MAX_STARTS           0x00000004
+#define CONFIG_GLOBALS_NO_FRIENDLY_WARNINGS 0x00000008
+#define CONFIG_GLOBALS_ALLMULTI             0x00000010
   int flags;
+  char destination;
   int auth_period;
   int held_period;
   int max_starts;
-  char allmulti;
-  char association;
-  struct config_string_list *allow_interfaces;
-  struct config_string_list *deny_interfaces;
-  
+  int stale_key_timeout;
 };
 
 
@@ -343,30 +332,33 @@ struct config_data
 
 
 int config_setup(char *);
-int config_get_priority(char *);
-struct config_network *config_build(char *);
+char config_build(char *);
 void config_destroy();
+struct config_network *config_get_network_config();
 int config_set_globals(struct interface_data *);
-char *config_get_startup_cmd();
-char *config_get_first_auth_cmd();
-char *config_get_reauth_cmd();
 char config_get_allmulti();
+char config_get_display_friendly_warnings();
 char config_get_association();
 char *config_get_logfile();
-struct config_string_list *config_allowed_interfaces();
-struct config_string_list *config_denied_interfaces();
+int config_get_ipc_gid();
+char config_get_destination();
+char *config_get_log_facility();
+void config_set_forced_profile(char *);
+char *config_get_default_int();
 
 int config_parse();
 int config_contains_network(char *);
 int config_allows_network(struct config_data *, char *);
 
-int config_int_in_list(char *, struct config_string_list *);
-
 // * private functions for config code
 void initialize_config_wpa_psk(struct config_wpa_psk **);
 void delete_config_wpa_psk(struct config_wpa_psk **);
 void dump_config_wpa_psk(struct config_wpa_psk *);
 
+void initialize_config_static_wep(struct config_static_wep **);
+void delete_config_static_wep(struct config_static_wep **);
+void dump_config_static_wep(struct config_static_wep *);
+
 void initialize_config_eap_tls(struct config_eap_tls **);
 void delete_config_eap_tls(struct config_eap_tls **);
 void dump_config_eap_tls(struct config_eap_tls *);
