@@ -1,78 +1,17 @@
-/**
- * A client-side 802.1x implementation supporting EAP/TLS
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- * 
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
  * EAP-MSCHAPv2 Function implementations
  * 
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * File: eapmschapv2.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eapmschapv2.c,v 1.23 2004/06/29 01:35:35 chessing Exp $
- * $Date: 2004/06/29 01:35:35 $
+ * $Id: eapmschapv2.c,v 1.29 2005/08/09 01:39:16 chessing Exp $
+ * $Date: 2005/08/09 01:39:16 $
  * $Log: eapmschapv2.c,v $
- * Revision 1.23  2004/06/29 01:35:35  chessing
- *
- * Added credit to eapleap.c for Gilbert Goodwill (who gave me the clues I needed to figure out the keying piece for LEAP).  Added patch from Toby Collett to make PEAP work for broken implementations of IAS.  (New option ias_quirk has been added.)
- *
- * Revision 1.22  2004/06/15 03:35:20  chessing
- *
- * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
- *
- * Revision 1.21  2004/06/15 03:22:29  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.29  2005/08/09 01:39:16  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
@@ -89,7 +28,6 @@
 #include "eapmschapv2.h"
 #include "mschapv2.h"
 #include "eap.h"
-#include "interactive.h"
 
 
 int eapmschapv2_setup(struct generic_eap_data *thisint)
@@ -324,9 +262,6 @@ int eapmschapv2_process(struct generic_eap_data *thisint, u_char *dataoffs,
     case MS_CHAPV2_FAILURE:
       debug_printf(DEBUG_NORMAL, "MS-CHAPv2 Authentication Failure!\n");
       *outsize = 0;
-      // We should probably process the failure info, and respond as needed,
-      // but, we really don't care if a failure is retryable, as 802.1x will
-      // just try again anyway. ;)
       break;
 
     case MS_CHAPV2_CHANGE_PWD:
@@ -340,11 +275,20 @@ int eapmschapv2_process(struct generic_eap_data *thisint, u_char *dataoffs,
 int eapmschapv2_get_keys(struct interface_data *thisint)
 {
   struct mschapv2_vars *myconf;
+  struct config_network *network_data;
 
-  if ((!thisint) || (!thisint->userdata) || (!thisint->userdata->activemethod) || (!thisint->userdata->activemethod->eap_data))
-      return XEMALLOC;
+  if (!thisint) return XEMALLOC;
 
-  myconf = (struct mschapv2_vars *)thisint->userdata->activemethod->eap_data;
+  network_data = config_get_network_config();
+  
+  if (network_data == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid network configuration structure! "
+		   "(%s:%d)\n", __FUNCTION__, __LINE__);
+      return XEBADCONFIG;
+    }
+
+  myconf = (struct mschapv2_vars *)network_data->activemethod->eap_data;
   if (thisint->keyingMaterial != NULL)
     {
       free(thisint->keyingMaterial);
