@@ -1,83 +1,24 @@
-/**
- * A client-side 802.1x implementation supporting EAP/TLS
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- * 
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
  * EAPPEAP Function implementations
  * 
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * File: eappeap.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eappeap.c,v 1.26 2004/07/15 04:15:37 chessing Exp $
- * $Date: 2004/07/15 04:15:37 $
+ * $Id: eappeap.c,v 1.34 2005/08/09 01:39:17 chessing Exp $
+ * $Date: 2005/08/09 01:39:17 $
  * $Log: eappeap.c,v $
- * Revision 1.26  2004/07/15 04:15:37  chessing
- *
- * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
- *
- * Revision 1.25  2004/06/15 03:35:20  chessing
- *
- * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
- *
- * Revision 1.24  2004/06/15 03:22:31  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.34  2005/08/09 01:39:17  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
 
 #include <string.h>
+#include <stdlib.h>
+
 #include "profile.h"
 #include "config.h"
 #include "xsup_debug.h"
@@ -88,7 +29,6 @@
 #include "eap.h"
 #include "eappeap.h"
 #include "peap_phase2.h"
-#include "interactive.h"
 
 
 int eappeap_setup(struct generic_eap_data *thisint)
@@ -167,7 +107,14 @@ int eappeap_setup(struct generic_eap_data *thisint)
       // We were told not to verify certificates.  Spew out a warning, and
       // then do it!
       mytls_vars->verify_mode = SSL_VERIFY_NONE;
-      debug_printf(DEBUG_NORMAL, "****WARNING**** Turning off certificate verification is a *VERY* bad idea!  You should not use this mode outside of basic testing, as it will compromise the security of your connection!\n");
+      
+      if (config_get_display_friendly_warnings())
+	{
+	  debug_printf(DEBUG_NORMAL, "****WARNING**** Turning off certificate "
+		       "verification is a *VERY* bad idea!  You should not "
+		       "use this mode outside of basic testing, as it will "
+		       "compromise the security of your connection!\n");
+	}
     } else {
       if (tls_funcs_load_root_certs(thisint, userdata->root_cert, 
 				    userdata->root_dir, userdata->crl_dir) != XENONE)
@@ -176,7 +123,7 @@ int eappeap_setup(struct generic_eap_data *thisint)
 	  return XETLSINIT;
 	}
     }
-
+  
   if ((userdata->user_cert != NULL) && ((userdata->user_key_pass != NULL) ||
 				   (thisint->tempPwd != NULL)))
     {
@@ -292,8 +239,12 @@ int eappeap_process(struct generic_eap_data *thisint, u_char *dataoffs,
   set_peap_version(p2d, peap_version);  // Tell PEAP what version we want to use.
   dataoffs[0] = ((uint8_t)dataoffs[0] & 0xfc);  // Mask out the version bits.
   
-  tls_funcs_decode_packet(thisint, dataoffs, insize,  outframe, outsize, 
-			  (phase2_call)peap_do_phase2, userdata->chunk_size);
+  if (tls_funcs_decode_packet(thisint, dataoffs, insize,  outframe, outsize, 
+			  (phase2_call)peap_do_phase2, 
+			      userdata->chunk_size) != XENONE)
+    {
+      return XEGENERROR;
+    }
 
   // We need to reset the version bits, just in case we store this frame for 
   // use later.
@@ -303,7 +254,7 @@ int eappeap_process(struct generic_eap_data *thisint, u_char *dataoffs,
     {
       debug_printf(DEBUG_AUTHTYPES, "Nothing returned from PEAP!\n");
       *outsize = 0;
-      return 0;
+      return XENONE;
     }
   
   // By the time we come out the first time, we should have decided on which
@@ -331,9 +282,19 @@ int eappeap_process(struct generic_eap_data *thisint, u_char *dataoffs,
 	  if (mytls_vars->sessionkeyconst == NULL) return XEMALLOC;
 	  
 	  bzero(mytls_vars->sessionkeyconst, PEAPv1_SESSION_KEY_CONST_SIZE);
-	  strncpy(mytls_vars->sessionkeyconst, PEAPv1_SESSION_KEY_CONST,
-		  PEAPv1_SESSION_KEY_CONST_SIZE);
-	  mytls_vars->sessionkeylen = PEAPv1_SESSION_KEY_CONST_SIZE;
+
+	  if (userdata->proper_peapv1 == 0)
+	    {
+	      strncpy(mytls_vars->sessionkeyconst, PEAP_SESSION_KEY_CONST,
+		      PEAP_SESSION_KEY_CONST_SIZE);
+	      mytls_vars->sessionkeylen = PEAP_SESSION_KEY_CONST_SIZE;
+	    } else {
+	      debug_printf(DEBUG_AUTHTYPES, "Using proper PEAPv1 key constant"
+			   "!\n");
+	      strncpy(mytls_vars->sessionkeyconst, PEAPv1_SESSION_KEY_CONST,
+		      PEAPv1_SESSION_KEY_CONST_SIZE);
+	      mytls_vars->sessionkeylen = PEAPv1_SESSION_KEY_CONST_SIZE;
+	    }
 	  break;
 	  
 	default:
@@ -345,24 +306,36 @@ int eappeap_process(struct generic_eap_data *thisint, u_char *dataoffs,
   if (*outsize > 0)
     {
       outframe[0] = outframe[0]+p2d->peap_version;
-    }
+    } 
 
   return XENONE;
 }
 
 int eappeap_get_keys(struct interface_data *thisint)
 {
+  struct config_network *network_data;
+
   if (!thisint)
     {
-      debug_printf(DEBUG_NORMAL, "Invalid interface struct passed to eappeap_get_keys()!\n");
+      debug_printf(DEBUG_NORMAL, "Invalid interface struct passed to %s()!\n",
+		   __FUNCTION__);
       return XEMALLOC;
     }
 
+  network_data = config_get_network_config();
+
+  if (network_data == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid network configuration struct! (%s:"
+		   "%d)\n", __FUNCTION__, __LINE__);
+      return XEBADCONFIG;
+    }
+
   if (thisint->keyingMaterial != NULL)
     {
       free(thisint->keyingMaterial);
     }
-  thisint->keyingMaterial = tls_funcs_gen_keyblock(thisint->userdata->activemethod);
+  thisint->keyingMaterial = tls_funcs_gen_keyblock(network_data->activemethod);
   thisint->keyingLength = 32;
 
   if (thisint->keyingMaterial == NULL) return -1;
