@@ -1,78 +1,17 @@
-/**
- * A client-side 802.1x implementation 
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- *
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
- * The driver function for a Linux application layer EAPOL 
- * implementation
+ * PEAP phase 2 implementation.
+ *
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * File: peap_phase2.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: peap_phase2.c,v 1.19 2004/07/15 04:15:37 chessing Exp $
- * $Date: 2004/07/15 04:15:37 $
+ * $Id: peap_phase2.c,v 1.24 2005/08/09 01:39:17 chessing Exp $
+ * $Date: 2005/08/09 01:39:17 $
  * $Log: peap_phase2.c,v $
- * Revision 1.19  2004/07/15 04:15:37  chessing
- *
- * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
- *
- * Revision 1.18  2004/06/29 01:35:36  chessing
- *
- * Added credit to eapleap.c for Gilbert Goodwill (who gave me the clues I needed to figure out the keying piece for LEAP).  Added patch from Toby Collett to make PEAP work for broken implementations of IAS.  (New option ias_quirk has been added.)
- *
- * Revision 1.17  2004/06/15 03:22:31  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.24  2005/08/09 01:39:17  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
@@ -246,7 +185,8 @@ void do_peap_version1(struct phase2_data *thisint, u_char *in, int in_size,
     default:
       if (eap_create_active_method(&thisint->activemethod,
 				   thisint->eapdata->identity,
-				   thisint->eapdata->tempPwd) != 0)
+				   thisint->eapdata->tempPwd,
+				   thisint->eapdata->intName) != 0)
 	{
 	  debug_printf(DEBUG_NORMAL, "Couldn't build active method!  Phase 2 authentication will not happen!\n");
 	  return;
@@ -335,7 +275,7 @@ void do_peap_version0(struct phase2_data *thisint, u_char *in, int in_size,
 }
 
 
-void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size, 
+int peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size, 
 		    u_char *out, int *out_size)
 {
   struct tls_vars *mytls_vars;
@@ -348,7 +288,7 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
   if ((!thisint) || (!in) || (!out))
     {
       debug_printf(DEBUG_NORMAL, "Invalid parameters passed in to peap_do_phase2()!\n");
-      return;
+      return XEGENERROR;
     }
 
   *out_size = 0;
@@ -358,7 +298,7 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
   if (mytls_vars == NULL)
     {
       debug_printf(DEBUG_NORMAL, "mytls_vars (thisint->eap_data) == NULL!\n");
-      return;
+      return XEGENERROR;
     }
 
   peapconf = (struct config_eap_peap *)thisint->eap_conf_data;
@@ -366,7 +306,7 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
   if (peapconf == NULL)
     {
       debug_printf(DEBUG_NORMAL, "peapconf == NULL!\n");
-      return;
+      return XEGENERROR;
     }
 
   p2d = (struct phase2_data *)mytls_vars->phase2data;
@@ -377,7 +317,7 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
       if (p2d->eapdata == NULL)
 	{
 	  *out_size = 0;
-	  return;
+	  return XEGENERROR;
 	}
       memset(p2d->eapdata, 0, sizeof(struct generic_eap_data));
       
@@ -385,14 +325,16 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
     }
 
   p2d->eapdata->eap_conf_data = peapconf->phase2;
-  p2d->eapdata->identity = thisint->identity;
+  // XXX Clean up!
+  if (peapconf->identity == NULL) debug_printf(DEBUG_NORMAL, "peapconf->identity is NULL!\n");
+  p2d->eapdata->identity = peapconf->identity;
   p2d->eapdata->ias_quirk = peapconf->ias_quirk;
 
   decr_data = (char *)malloc(1550);
   if (decr_data == NULL) 
     {
       debug_printf(DEBUG_NORMAL, "Couldn't allocate memory for decryption buffer!\n");
-      return;
+      return XEMALLOC;
     }
 
   encr_data = (char *)malloc(1550);
@@ -400,12 +342,20 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
     {
       debug_printf(DEBUG_NORMAL, "Couldn't allocate memory for encryption buffer!\n");
       free(decr_data);
-      return;
+      return XEMALLOC;
     }
 
   if (in_size > 0)
     {
-      tls_crypt_decrypt(thisint, in, in_size, decr_data, &decrsize);
+      if (tls_crypt_decrypt(thisint, in, in_size, decr_data, &decrsize) 
+	  != XENONE)
+	{
+	  debug_printf(DEBUG_AUTHTYPES, "Decryption failed.\n");
+	  bzero(in, 50);  // Zero enough to keep us from processing this
+	                  // again.
+	  *out_size = 0;
+	  return XETLSCRYPTFAIL;
+	}
 
       debug_printf(DEBUG_AUTHTYPES, "Decrypted dump : \n");
       debug_hex_dump(DEBUG_AUTHTYPES, decr_data, decrsize);
@@ -422,7 +372,7 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
       *out_size = 1;
       free(decr_data);
       free(encr_data);
-      return;
+      return XENONE;
     }
 
   debug_printf(DEBUG_AUTHTYPES, "Decrypted packet returned %d byte(s)\n", decrsize);
@@ -474,6 +424,8 @@ void peap_do_phase2(struct generic_eap_data *thisint, u_char *in, int in_size,
 
   free(encr_data);
   free(decr_data);
+
+  return XENONE;
 }
 
 
