@@ -1,79 +1,24 @@
-/**
- * A client-side 802.1x implementation supporting EAP/TLS
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- * 
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
  * EAPTLS (RFC 2716) Function implementations
  * 
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * File: eaptls.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eaptls.c,v 1.27 2004/06/15 03:35:21 chessing Exp $
- * $Date: 2004/06/15 03:35:21 $
+ * $Id: eaptls.c,v 1.34 2005/08/09 01:39:18 chessing Exp $
+ * $Date: 2005/08/09 01:39:18 $
  * $Log: eaptls.c,v $
- * Revision 1.27  2004/06/15 03:35:21  chessing
- *
- * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
- *
- * Revision 1.26  2004/06/15 03:22:31  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.34  2005/08/09 01:39:18  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
 
 #include <string.h>
+#include <stdlib.h>
+
 #include "profile.h"
 #include "config.h"
 #include "xsup_debug.h"
@@ -82,7 +27,6 @@
 #include "eap_types/tls/eaptls.h"
 #include "eap_types/tls/tls_funcs.h"
 #include "eap.h"
-#include "interactive.h"
 
 int eaptls_setup(struct generic_eap_data *thisint)
 {
@@ -106,10 +50,12 @@ int eaptls_setup(struct generic_eap_data *thisint)
   if (!thisint->eap_data) return XEMALLOC;
 
   mytls_vars = (struct tls_vars *)thisint->eap_data;
+  bzero(mytls_vars, sizeof(struct tls_vars));
 
   // Set our variables to NULL.
   mytls_vars->ctx = NULL;
   mytls_vars->ssl = NULL;
+  mytls_vars->engine = NULL;
   mytls_vars->ssl_in = NULL;
   mytls_vars->ssl_out = NULL;
   mytls_vars->tlsoutdata = NULL;
@@ -254,18 +200,29 @@ int eaptls_process(struct generic_eap_data *thisint, u_char *dataoffs,
 
 int eaptls_get_keys(struct interface_data *thisint)
 {
+  struct config_network *network_data;
+
   if (!thisint)
     {
       debug_printf(DEBUG_NORMAL, "Invalid interface struct passed to eaptls_get_keys()!\n");
       return -1;
     }
 
+  network_data = config_get_network_config();
+
+  if (network_data == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid network configuration struct! (%s:"
+		   "%d)\n", __FUNCTION__, __LINE__);
+      return XEBADCONFIG;
+    }
+
   if (thisint->keyingMaterial != NULL)
     {
       free(thisint->keyingMaterial);
     }
 
-  thisint->keyingMaterial = tls_funcs_gen_keyblock(thisint->userdata->activemethod);
+  thisint->keyingMaterial = tls_funcs_gen_keyblock(network_data->activemethod);
   thisint->keyingLength = 32;
   
   if (thisint->keyingMaterial == NULL) return -1;
