@@ -1,74 +1,17 @@
-/**
- * A client-side 802.1x implementation supporting EAP/TLS
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- *
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
- * En/Decrypt Function implementations
+ * TLS En/Decrypt Function implementations
+ *
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
  *
  * File: tls_crypt.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: tls_crypt.c,v 1.10 2004/06/17 22:56:59 chessing Exp $
- * $Date: 2004/06/17 22:56:59 $
+ * $Id: tls_crypt.c,v 1.14 2005/08/09 01:39:18 chessing Exp $
+ * $Date: 2005/08/09 01:39:18 $
  * $Log: tls_crypt.c,v $
- * Revision 1.10  2004/06/17 22:56:59  chessing
- *
- * Fixed the problem where an attempt to execute an invalid program would result in a second instance of XSupplicant running.  (Which would then cause all kinds of interesting problems. ;)  Also added a bunch of additional debugging to the TLS code to try to resolve the strange stalls that sometimes happen in authentications that use TLS for phase 1.
- *
- * Revision 1.9  2004/06/15 03:22:32  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.14  2005/08/09 01:39:18  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
@@ -127,8 +70,8 @@ u_char *tls_crypt_gen_keyblock(struct generic_eap_data *thisint, char *sesskey,
   memcpy(p, mytls_vars->ssl->s3->client_random, SSL3_RANDOM_SIZE);
   p+= SSL3_RANDOM_SIZE;
   memcpy(p, mytls_vars->ssl->s3->server_random, SSL3_RANDOM_SIZE);
-  tls_funcs_PRF(mytls_vars->ssl->session->master_key, 
-		mytls_vars->ssl->session->master_key_length,
+  tls_funcs_PRF(SSL_get_session(mytls_vars->ssl)->master_key, 
+		SSL_get_session(mytls_vars->ssl)->master_key_length,
 		sesskey, sesskeylen, seed, 
 		SSL3_RANDOM_SIZE * 2, retblock, 
 		TLS_SESSION_KEY_SIZE);
@@ -157,7 +100,7 @@ int tls_crypt_decrypt(struct generic_eap_data *thisint, u_char *in_data, int in_
     {
       debug_printf(DEBUG_NORMAL, "In tls_crypt.c, BIO_reset(mytls_vars->ssl_in) failed.\n");
       tls_funcs_process_error();
-      return -1;
+      return XETLSCRYPTFAIL;
     }
 
   rc=BIO_write(mytls_vars->ssl_in, in_data, in_size);
@@ -166,7 +109,7 @@ int tls_crypt_decrypt(struct generic_eap_data *thisint, u_char *in_data, int in_
     {
       debug_printf(DEBUG_NORMAL, "In tls_crypt.c, BIO_reset(mytls_vars->ssl_out) failed.\n");
       tls_funcs_process_error();
-      return -1;
+      return XETLSCRYPTFAIL;
     }
 
   rc=SSL_read(mytls_vars->ssl, out_data, 1000);
@@ -174,7 +117,7 @@ int tls_crypt_decrypt(struct generic_eap_data *thisint, u_char *in_data, int in_
     {
       debug_printf(DEBUG_NORMAL, "In tls_crypt.c, SSL_read(mytls_vars->ssl, out_data, 1000) failed.\n");
       tls_funcs_process_error();
-      return -1;
+      return XETLSCRYPTFAIL;
     }
 
   *out_size = rc;
