@@ -1,86 +1,31 @@
-/**
- * A client-side 802.1x implementation supporting EAP/TLS
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- * 
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
  * EAPTLS (RFC 2716) Function implementations
  * 
- * File: eaptls.c
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
+ * File: tls_funcs.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: tls_funcs.c,v 1.30 2004/07/15 04:15:37 chessing Exp $
- * $Date: 2004/07/15 04:15:37 $
+ * $Id: tls_funcs.c,v 1.39 2005/08/12 03:34:06 chessing Exp $
+ * $Date: 2005/08/12 03:34:06 $
  * $Log: tls_funcs.c,v $
- * Revision 1.30  2004/07/15 04:15:37  chessing
+ * Revision 1.39  2005/08/12 03:34:06  chessing
+ * Fix to the TLS implementation, should help avoid some of the weird 'block cipher pad' errors.  Also includes a partial implementation of the ability to use static WEP keys based on the SSID in use.
  *
- * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
- *
- * Revision 1.29  2004/06/17 22:56:59  chessing
- *
- * Fixed the problem where an attempt to execute an invalid program would result in a second instance of XSupplicant running.  (Which would then cause all kinds of interesting problems. ;)  Also added a bunch of additional debugging to the TLS code to try to resolve the strange stalls that sometimes happen in authentications that use TLS for phase 1.
- *
- * Revision 1.28  2004/06/15 03:22:32  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.38  2005/08/09 01:39:18  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
 
 #include <openssl/ssl.h>
+#include <openssl/engine.h>
+#include <openssl/ui.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
 #include <openssl/hmac.h>
+#include <openssl/md5.h>
 #include <netinet/in.h>
 #include <string.h>
 #include <inttypes.h>
@@ -93,6 +38,10 @@
 #include "xsup_debug.h"
 #include "xsup_err.h"
 
+int engine_load_dynamic_opensc(struct smartcard *sc);
+void set_smartcard_pin(char *pin);
+UI_METHOD *UI_noninteractive(void);
+
 char *get_cert_common_name(SSL *ssl_ctx)
 {
   char *commonName = NULL;
@@ -172,11 +121,14 @@ static int ssl_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 int tls_funcs_init(struct generic_eap_data *thisint)
 {
   struct tls_vars *mytls_vars;
+  struct config_eap_tls *userdata;
 
   if (thisint == NULL) return XETLSINIT;
   if (thisint->eap_data == NULL) return XETLSINIT;
+  if (thisint->eap_conf_data == NULL) return XETLSINIT;
 
   mytls_vars = (struct tls_vars *)thisint->eap_data;
+  userdata = (struct config_eap_tls *)thisint->eap_conf_data;
 
   SSL_library_init();
   SSL_load_error_strings();
@@ -189,14 +141,64 @@ int tls_funcs_init(struct generic_eap_data *thisint)
       return XETLSINIT;
     }
 
+  // XXX Clean up this code.
+  if (thisint->eapNum == EAP_TYPE_TLS)
+    {
+      if ( userdata->sc.engine_id != NULL )
+	{
+	  if(!engine_load_dynamic_opensc(&userdata->sc))
+	    {
+	      debug_printf(DEBUG_NORMAL, "OpenSC Engine will be unavailable!\n");
+	    }
+	  
+	  debug_printf(DEBUG_NORMAL, "Using Engine with ID \"%s\"\n", userdata->sc.engine_id);
+	  
+	  mytls_vars->engine = ENGINE_by_id(userdata->sc.engine_id);
+	  if (!mytls_vars->engine)
+	    {
+	      debug_printf(DEBUG_NORMAL, "Engine not available!\n");
+	      /* This error will be unrecoverable - XSupplicant should die */
+	      return XETLSINIT;
+	    }
+	  
+	  /* Now everything that can be done without having a smartcard plugged in
+	   * is done. The next step is initializing the Engine. 
+	   * This step should probably be done in some place where it can be 
+	   * retried if it fails.
+	   */
+	  
+	  if (!ENGINE_init(mytls_vars->engine))
+	    {
+	      debug_printf(DEBUG_NORMAL, "Can't initialize OpenSC Engine! "
+			   "Is the smartcard plugged in?\n");
+	      /* If we get this error there's probably no smartcard connected -
+	       * we should be able to retry the call to ENGINE_init() */
+	      return XETLSINIT;
+	    }
+	}
+    }
+
   return XENONE;
 }
 
+int tls_funcs_build_new_session(struct tls_vars *mytls_vars)
+{
+  mytls_vars->ssl = SSL_new(mytls_vars->ctx);
+  if (!mytls_vars->ssl)
+    {
+      debug_printf(DEBUG_NORMAL, "Couldn't create SSL object!\n");
+      tls_funcs_process_error();
+      
+      return XETLSSTARTFAIL;
+    }
+  return XENONE;
+}
+  
 int tls_funcs_start(struct tls_vars *mytls_vars)
 {
   SSL_SESSION *sess = NULL;
   unsigned long err;
-  int counter;
+  int counter, resval = XENONE;
 
   if (mytls_vars == NULL)
     {
@@ -208,14 +210,8 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
 
   if (!mytls_vars->ssl)
     {
-      mytls_vars->ssl = SSL_new(mytls_vars->ctx);
-      if (!mytls_vars->ssl)
-	{
-	  debug_printf(DEBUG_NORMAL, "Couldn't create SSL object!\n");
-	  tls_funcs_process_error();
-
-	  return XETLSSTARTFAIL;
-	}
+      resval = tls_funcs_build_new_session(mytls_vars);
+      if (resval != XENONE) return resval;
     } else {
       // We already established a connection, so we probably we need to
       // resume the session.
@@ -226,36 +222,43 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
 	    {
 	      debug_printf(DEBUG_AUTHTYPES, "Couldn't get session information!  We won't try to resume this session!\n");
 	      mytls_vars->resuming = 0;
+
+	      // Clear the old session data.
+	      SSL_free(mytls_vars->ssl);
+
+	      // Set up a new session.
+	      resval = tls_funcs_build_new_session(mytls_vars);
+	      if (resval != XENONE) return resval;
 	    } else {
 	      debug_printf(DEBUG_AUTHTYPES, "Got session information, trying to resume session!\n");
 	      mytls_vars->resuming = 1;
+
+	      // We don't want to send an alert to the other end..  So do a 
+	      // quiet shutdown.  This violates the TLS standard, but it is 
+	      // needed to avoid confusing the other end of the connection 
+	      // when we want to do a reconnect!
+	      SSL_set_quiet_shutdown(mytls_vars->ssl, 1);
+	      
+	      // Now, close off our old session.
+	      err = 0;
+	      counter = 0;
+	      while ((err == 0) && (counter < 60))
+		{
+		  err = SSL_shutdown(mytls_vars->ssl);
+		  if (err == 0)
+		    {
+		      sleep(1);
+		      counter++;
+		    }
+		}
+
+	      if (err < 0)
+		{
+		  debug_printf(DEBUG_NORMAL, "Error trying to shut down SSL context data.\n");
+		  tls_funcs_process_error();
+		}
 	    }
 	}
-
-      // We don't want to send an alert to the other end..  So do a quiet
-      // shutdown.  This violates the TLS standard, but it needed to avoid
-      // confusing the other end of the connection when we want to do a
-      // reconnect!
-      SSL_set_quiet_shutdown(mytls_vars->ssl, 1);
-
-      // Now, close off our old session.
-      err = 0;
-      counter = 0;
-      while ((err == 0) && (counter < 60))
-	{
-	  err = SSL_shutdown(mytls_vars->ssl);
-	  if (err == 0)
-	    {
-	      sleep(1);
-	      counter++;
-	    }
-	}
-
-      if (err < 0)
-	{
-	  debug_printf(DEBUG_NORMAL, "Error trying to shut down SSL context data.\n");
-	  tls_funcs_process_error();
-	}
     }
 
   mytls_vars->ssl_in = BIO_new(BIO_s_mem());
@@ -302,7 +305,7 @@ int tls_funcs_parse(struct generic_eap_data *thisint, u_char *indata,
   BUF_MEM *retData;
   struct tls_vars *mytls_vars;
   char *retVal;
-  uint64_t length;
+  uint32_t length;
 
   if (!thisint)
     {
@@ -560,7 +563,12 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
 	  
 	  if ((mytls_vars->resuming != 1) || (mytls_vars->quickResponse != TRUE))
 	    {
-	      (*dophase2)(thisint, tlsptr, (insize-tlsindex), outframe, outsize);
+	      if (((*dophase2)(thisint, tlsptr, (insize-tlsindex), outframe, 
+			       outsize)) != XENONE)
+		{
+		  debug_printf(DEBUG_NORMAL, "Phase 2 failure!\n");
+		  return XEPHASE2FAILURE;
+		}
 	    } else {
 	      if (*outsize == 0)
 		{
@@ -634,7 +642,12 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
 
 	      if ((mytls_vars->resuming != 1) || (mytls_vars->quickResponse != TRUE))
 		{
-		  (*dophase2)(thisint, tlsptr, (insize-tlsindex), outframe, outsize);
+		  if ((*dophase2)(thisint, tlsptr, (insize-tlsindex), 
+				  outframe, outsize) != XENONE)
+		    {
+		      debug_printf(DEBUG_NORMAL, "Phase 2 Failure.\n");
+		      return XEPHASE2FAILURE;
+		    }
 		} else {
 		  if (*outsize == 0)
 		    {
@@ -752,7 +765,7 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
     {
       debug_printf(DEBUG_NORMAL, "Failed to initalize path to root certificate!\n");
       tls_funcs_process_error();
-      //      debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
+
       if(mytls_vars->ctx)
 	{
 	  SSL_CTX_free(mytls_vars->ctx);
@@ -841,6 +854,7 @@ int tls_funcs_load_user_cert(struct generic_eap_data *thisint,
 			     char *random_file)
 {
   struct tls_vars *mytls_vars;
+  struct config_eap_tls *userdata;
 
   if ((!thisint) || (!client_cert) || (!key_file) || (!password))
     {
@@ -855,6 +869,7 @@ int tls_funcs_load_user_cert(struct generic_eap_data *thisint,
     }
 
   mytls_vars = (struct tls_vars *)thisint->eap_data;
+  userdata = (struct config_eap_tls *)thisint->eap_conf_data;
 
   SSL_CTX_set_default_passwd_cb_userdata(mytls_vars->ctx, password);
   SSL_CTX_set_default_passwd_cb(mytls_vars->ctx, return_password);
@@ -875,10 +890,19 @@ int tls_funcs_load_user_cert(struct generic_eap_data *thisint,
     }
 
   debug_printf(DEBUG_CONFIG, "Loading user Private Key from %s...\n", key_file);
-
-  if (SSL_CTX_use_PrivateKey_file(mytls_vars->ctx, key_file, 
+  if (userdata->sc.engine_id)
+    {
+      EVP_PKEY *pkey;
+      debug_printf(DEBUG_CONFIG, "Loading user Private Key with id %s from %s...\n", userdata->sc.key_id, userdata->sc.engine_id);
+      set_smartcard_pin(password);
+      pkey = ENGINE_load_private_key(mytls_vars->engine, userdata->sc.key_id,
+		      UI_noninteractive(), NULL);
+      SSL_CTX_use_PrivateKey(mytls_vars->ctx, pkey);
+      //EVP_PKEY_free(pkey);
+    }
+  else if (SSL_CTX_use_PrivateKey_file(mytls_vars->ctx, key_file, 
 				  SSL_FILETYPE_PEM) != 1 &&
-      SSL_CTX_use_PrivateKey_file(mytls_vars->ctx, key_file, 
+           SSL_CTX_use_PrivateKey_file(mytls_vars->ctx, key_file, 
 				  SSL_FILETYPE_ASN1) != 1) 
     {
       tls_funcs_process_error();
@@ -908,6 +932,7 @@ int tls_funcs_load_user_cert(struct generic_eap_data *thisint,
     {
       return XETLSCERTLOAD;
     }
+  debug_printf(DEBUG_CONFIG, "Loaded random data from %s...\n", random_file);
 
   // If we made it this far, our user cert is loaded, so indicate it.
   mytls_vars->cert_loaded = TRUE;
@@ -964,7 +989,14 @@ int tls_funcs_cleanup(struct generic_eap_data *thisint)
   mytls_vars = (struct tls_vars *)thisint->eap_data;
 
   err = tls_funcs_failed(thisint);
-
+  if (mytls_vars->engine)
+    {
+      debug_printf(DEBUG_NORMAL, "Cleaning up OpenSSL Engine\n");
+      ENGINE_finish(mytls_vars->engine);
+      mytls_vars->engine = NULL;
+      debug_printf(DEBUG_NORMAL, "Cleaning up OpenSSL Engine internal ressources\n");
+      ENGINE_cleanup();
+    }
   if (mytls_vars->sessionkeyconst != NULL)
     {
       debug_printf(DEBUG_AUTHTYPES, "(EAP-TLS) Freeing session key const!\n");
@@ -1091,3 +1123,105 @@ tls_funcs_PRF(u_char *secret, int secret_len, u_char *label, int label_len,
   return retVal;
 }
 
+/* smartcard support */
+
+#define OPENSC_ENGINE_SO_PATH "/usr/lib/opensc/engine_opensc.so"
+#define OPENSC_ENGINE_ID      "opensc"
+
+/* This function 
+ * - loads OpenSSL's "dynamic" engine
+ * - executes all the commands given in the pre array of strings
+ *   These commands will usually load the shared object, do some 
+ *   initialization and add the engine to OpenSSL's internal list of 
+ *   Engines
+ */
+int engine_load_dynamic(char *pre[])
+{
+  char *engine_id = "dynamic";
+  int rc;
+  ENGINE *e;
+  ENGINE_load_dynamic();
+  e = ENGINE_by_id(engine_id);
+  if(!e)
+    {
+      printf("can't find engine %s\n", engine_id);
+      goto err;
+    }
+  while(pre && pre[0])
+    {
+      /*printf("\"%s\" \"%s\"\n", pre[0], pre[1]);*/
+      rc = ENGINE_ctrl_cmd_string(e, pre[0], pre[1], 0);
+      if(rc == 0)
+        {
+          printf("ctrl cmd_string failed: %s %s\n", 
+					pre[0], pre[1]);
+  	  goto err_pre;
+        }
+      pre += 2;
+    }
+  /* Free the reference to the "dynamic" engine
+   * The OpenSC engine can still be looked up using 
+   * ENGINE_by_id() */
+  ENGINE_free(e);
+  return 1;
+err_pre:
+  ENGINE_free(e);
+err:
+  ENGINE_cleanup();
+  return 0;
+}
+
+/* This function
+ *  - makes the opensc engine available to OpenSSL
+ */
+int engine_load_dynamic_opensc(struct smartcard *sc)
+{
+  debug_printf(DEBUG_NORMAL, "Loading opensc engine.\n");
+  if(!sc->opensc_so_path)
+    {
+      /* use the default value */
+      sc->opensc_so_path = OPENSC_ENGINE_SO_PATH;
+    }
+  char *pre_cmd[] = 
+    {
+      "SO_PATH", sc->opensc_so_path,
+      "ID", OPENSC_ENGINE_ID,
+      "LIST_ADD", "1",
+      "LOAD", NULL,
+      NULL, NULL
+    };
+  return engine_load_dynamic(pre_cmd);
+}
+
+/* provide a UI_METHOD that makes it possible to use a string as the
+ * smartcard PIN */
+char *smartcard_pin = NULL;
+
+void set_smartcard_pin(char *pin)
+{
+  smartcard_pin = pin;
+}
+
+void unset_smartcard_pin()
+{
+  set_smartcard_pin(NULL);
+}
+
+int read_string(UI *ui, UI_STRING *uis)
+{
+  if(smartcard_pin)
+    {
+      UI_set_result(ui, uis, smartcard_pin);
+      return 1;
+    }
+  return 0;
+}
+
+UI_METHOD *UI_noninteractive(void)
+{
+  UI_METHOD *ui_method;
+  ui_method = UI_create_method("ui_noninteractive");
+  UI_method_set_reader(ui_method, read_string);
+  return ui_method;
+}
+
