@@ -1,70 +1,17 @@
-/**
- * A client-side 802.1x implementation supporting EAP/TLS
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- *
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
  * EAPTTLS Phase 2 Function implementations
  *
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * File: ttlsphase2.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: ttlsphase2.c,v 1.18 2004/06/15 03:22:32 chessing Exp $
- * $Date: 2004/06/15 03:22:32 $
+ * $Id: ttlsphase2.c,v 1.25 2005/08/09 01:39:18 chessing Exp $
+ * $Date: 2005/08/09 01:39:18 $
  * $Log: ttlsphase2.c,v $
- * Revision 1.18  2004/06/15 03:22:32  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.25  2005/08/09 01:39:18  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
@@ -81,9 +28,9 @@
 #include "config.h"
 #include "profile.h"
 #include "eap.h"
-#include "interactive.h"
 #include "../tls/tls_crypt.h"
 #include "xsup_debug.h"
+#include "xsup_err.h"
 #include "../mschapv2/mschapv2.h"
 #include "ttlsphase2.h"
 
@@ -250,7 +197,7 @@ void ttls_do_mschapv2(struct generic_eap_data *thisint, char *out_data, int *out
     }
 
   memcpy(&mschap_challenge, challenge, 16);
-  id = challenge[17];
+  id = challenge[16];
 
   // Send the MS-CHAP AVP
   build_avp(MS_CHAP_CHALLENGE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (char *)&mschap_challenge, 16, &out_data[avp_offset], &avp_out_size);
@@ -279,7 +226,7 @@ void ttls_do_mschapv2(struct generic_eap_data *thisint, char *out_data, int *out
 // For phase 2 MS-CHAP, we get 8 bytes implicit challenge, and 1 byte for ID.
 void ttls_do_mschap(struct generic_eap_data *thisint, char *out_data, int *out_size)
 {
-  u_char mschap_challenge[8], mschap_answer[49];
+  u_char mschap_challenge[8], mschap_answer[50];
   u_char mschap_result[24];
   char *username = NULL, *password = NULL, *challenge = NULL;
   int avp_offset, avp_out_size, username_size, id;
@@ -349,7 +296,7 @@ void ttls_do_mschap(struct generic_eap_data *thisint, char *out_data, int *out_s
     }
 
   memcpy((char *)&mschap_challenge[0], challenge, 8);
-  id = challenge[9];
+  id = challenge[8];
 
   // Send the MS-CHAP AVP
   build_avp(MS_CHAP_CHALLENGE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (char *)&mschap_challenge, 8, &out_data[avp_offset], &avp_out_size);
@@ -510,9 +457,6 @@ void ttls_do_chap(struct generic_eap_data *thisint, char *out_data, int *out_siz
 void ttls_do_bogus(struct generic_eap_data *thisint, char *out_data, int *out_size)
 {
   debug_printf(DEBUG_NORMAL, "Attempting to call an undefined Phase 2!\n");
-
-  // We probably really don't want to die here.  We need to reconsider.
-  exit(255);
 }
 
 void ttls_do_pap(struct generic_eap_data *thisint, char *out_data, int *out_size)
@@ -608,25 +552,29 @@ void ttls_do_pap(struct generic_eap_data *thisint, char *out_data, int *out_size
 }
 
 // We don't do anything with the "in" stuff for now..
-void ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, char *out, int *out_size)
+int ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, char *out, int *out_size)
 {
-  int toencsize, i;
+  int toencsize, i, decrsize;
   char *toencout;
   struct config_eap_ttls *userdata;
   struct config_ttls_phase2 *phase2data;
+  char decr_data[1550];
 
   if ((!thisint) || (!thisint->eap_conf_data) || (!out))
     {
       debug_printf(DEBUG_NORMAL, "Invalid data pased in to ttls_do_phase2()!\n");
-      return;
+      return XEGENERROR;
     }
 
+  debug_printf(DEBUG_AUTHTYPES, "Encrypted Inner (%d) : \n", in_size);
+  debug_hex_dump(DEBUG_AUTHTYPES, in, in_size);
+
   userdata = (struct config_eap_ttls *)thisint->eap_conf_data;
 
   if (!userdata->phase2)
     {
       debug_printf(DEBUG_NORMAL, "Invalid userdata in ttls_do_phase2()!\n");
-      return;
+      return XEGENERROR;
     }
 
   phase2data = (struct config_ttls_phase2 *)userdata->phase2;
@@ -635,7 +583,32 @@ void ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, cha
   if (toencout == NULL)
     {
       debug_printf(DEBUG_NORMAL, "Couldn't allocate memory needed for encryption!\n");
-      return;
+      return XEMALLOC;
+    }
+
+  // This is a hack. :-(  It is needed for TTLS-MS-CHAPv2.
+  // XXX Fix better!
+  // A better fix would be to figure out how to have OpenSSL tell us how much
+  // of the packet it processed, so we can move our offset pointer to the end
+  // of the data that has been used.  This would result in a NULL packet
+  // being passed in for the first part, and the encrypted packet being
+  // passed in the second part.  This would allow us to process the packets
+  // "correctly" inside of the specific phase 2 handler.
+  if ((in_size > 0) && (in[0] != 0x14))
+    {
+      // We have something to decrypt!
+      tls_crypt_decrypt(thisint, in, in_size, decr_data, &decrsize);
+
+      debug_printf(DEBUG_AUTHTYPES, "Decrypted Inner (%d) : \n", in_size);
+      debug_hex_dump(DEBUG_AUTHTYPES, decr_data, decrsize);
+
+      if (decr_data[0] == 0x00)
+	{
+	  debug_printf(DEBUG_AUTHTYPES, "(Hack) Acking for second inner phase packet!\n");
+	  out[0] = 0x00;  // ACK
+	  *out_size = 1;
+	  return XENONE;
+	}
     }
 
   toencsize = 1550;
@@ -662,7 +635,7 @@ void ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, cha
     {
       *out_size = 0;
       free(toencout);
-      return;
+      return XENONE;
     }
 
   tls_crypt_encrypt_nolen(thisint, toencout, toencsize, out, out_size);
@@ -670,6 +643,7 @@ void ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, cha
 
   debug_printf(DEBUG_AUTHTYPES, "Returning from (TTLS) do_phase2 : \n");
   debug_hex_dump(DEBUG_AUTHTYPES, out, *out_size);
+  return XENONE;
 }
 
 
