@@ -1,85 +1,24 @@
-/**
- * A client-side 802.1x implementation
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- *
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
  * Handle keying for type 1 (RC4, non-TKIP) EAPOL Keys
  * File: eapol_key_type1.c
  *
+ * Licensed under a dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eapol_key_type1.c,v 1.15 2004/07/15 04:15:35 chessing Exp $
- * $Date: 2004/07/15 04:15:35 $
+ * $Id: eapol_key_type1.c,v 1.32 2005/08/09 01:39:14 chessing Exp $
+ * $Date: 2005/08/09 01:39:14 $
  * $Log: eapol_key_type1.c,v $
- * Revision 1.15  2004/07/15 04:15:35  chessing
- *
- * True/false int values are now bit flags in a byte.  PEAP now calls back in to functions from eap.c for phase 2 methods.  This allows for any phase 1 EAP type to work.  This resulted in some major changes the functions in eap.c, and in peap_phase2.c.  PEAP has been tested using both MS-CHAPv2, and EAP-MD5 as inner types.  More types need to be tested, and enabled.
- *
- * Revision 1.14  2004/06/15 03:35:19  chessing
- *
- * New updates including fixes to LEAP (keying now works with wireless) and adding EAP-AKA.
- *
- * Revision 1.13  2004/06/15 03:22:29  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.32  2005/08/09 01:39:14  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
 
 #include <stdio.h>
 #include <openssl/hmac.h>
+#include <openssl/rc4.h>
 #include <string.h>
-//#include <stdint.h>
 #include <netinet/in.h>
 #include "xsup_debug.h"
 #include "xsup_err.h"
@@ -87,11 +26,13 @@
 #include "cardif/cardif.h"
 #include "key_statemachine.h"
 #include "eapol_key_type1.h"
+#include "config.h"
+#include "timer.h"
 
 int eapol_dump_keydata(char *inframe, int framesize)
 {
   struct key_packet *keydata;
-  int length;
+  uint16_t length;
 
   if (!inframe)
     {
@@ -228,6 +169,82 @@ int eapol_key_type1_get_rc4(struct interface_data *thisint, u_char *enckey,
   return XENONE;
 }
 
+/*********************************
+ *
+ * If our rekey timer expires, we should quietly remove it.  In the case of
+ * this timer, we WANT it to expire with no events!  (Otherwise, the card
+ * in use may have issues.)
+ *
+ *********************************/
+void eapol_key_type1_clear_timer(struct interface_data *intdata)
+{
+  debug_printf(DEBUG_EVERYTHING, "Clearing rekey problem timer.  (This is "
+	       "harmless!)\n");
+  timer_cancel(REKEY_PROB_TIMER);
+}
+
+/*********************************
+ *
+ * Set up our timer to warn if the driver may have the card reset issue.
+ *
+ *********************************/
+void eapol_key_type1_set_rekey_prob_timer()
+{
+  if (timer_check_existing(REKEY_PROB_TIMER))
+    {
+      debug_printf(DEBUG_NORMAL, "Only %d seconds have elapsed since the last "
+		   "key was set!  Either your AP has the rekey interval set "
+		   "dangerously low, or your card may reset everytime new keys"
+		   " are set!  Please check the rekey interval time on your "
+		   "AP and report your card type, driver, and driver version "
+		   "number to the list!\n", REKEY_PROB_TIMEOUT);
+      timer_reset_timer_count(REKEY_PROB_TIMER, REKEY_PROB_TIMEOUT);
+    } else {
+      // There isn't an existing counter, so set up a new one.
+      timer_add_timer(REKEY_PROB_TIMER, REKEY_PROB_TIMEOUT, NULL,
+		      &eapol_key_type1_clear_timer);
+    }
+}
+
+/*********************************
+ *
+ * Display the stale key warning, and disable the timer that was running.
+ *
+ *********************************/
+void eapol_key_type1_stale_key_warn(struct interface_data *intdata)
+{
+  debug_printf(DEBUG_NORMAL, "Your unicast key has been in use for %d "
+	       "minute(s).  Given the time it takes to crack WEP keys, your "
+	       "data is now less secure than it was.  You should consider "
+	       "asking your network administrator to lower the rekey interval"
+	       " for your wireless network.\n\n"
+	       " *** PLEASE NOTE ***  This is just a warning message, and does"
+	       " not indicate that your WEP key has been broken!\n", 
+	       (config_get_stale_key_timeout()/60));
+  timer_cancel(STALE_KEY_WARN_TIMER);
+}
+
+/*********************************
+ *
+ * Set a timer that watches how long a key has been in use.  (Should only
+ * be used for unicast keys, since broadcast keys aren't very secure to
+ * begin with!)  If the timer expires, then we need to warn the user that
+ * their security may be weaker than it used to be.
+ *
+ *********************************/
+void eapol_key_type1_set_stale_key_warn_timer()
+{
+  if (timer_check_existing(STALE_KEY_WARN_TIMER))
+    {
+      timer_reset_timer_count(STALE_KEY_WARN_TIMER, 
+			      config_get_stale_key_timeout());
+    } else {
+      // Set up a new warning counter.
+      timer_add_timer(STALE_KEY_WARN_TIMER, config_get_stale_key_timeout(), 
+		      NULL, &eapol_key_type1_stale_key_warn);
+    }
+}
+
 /*********************************
  *
  * Decrypt the key, and set it on the interface.  If there isn't a key to
@@ -238,7 +255,8 @@ int eapol_key_type1_decrypt(struct interface_data *thisint, char *inframe,
 			    int framesize)
 {
   struct key_packet *keydata = NULL;
-  int keylen, rc=0, length;
+  int keylen, rc=0;
+  uint16_t length;
   u_char *newkey = NULL, *enckey = NULL;
 
   if ((!thisint) || (!inframe))
@@ -249,7 +267,6 @@ int eapol_key_type1_decrypt(struct interface_data *thisint, char *inframe,
 
   keydata = (struct key_packet *)&inframe[0];
 
-  //  keylen = framesize - sizeof(*keydata) - 2;
   memcpy(&length, keydata->key_length, 2);
   keylen = ntohs(length);
 
@@ -257,7 +274,6 @@ int eapol_key_type1_decrypt(struct interface_data *thisint, char *inframe,
 	       keydata->key_index & UNICAST_KEY ? "unicast" : "broadcast",
 	       (keydata->key_index & KEY_INDEX)+1, keylen);
 
-  //  debug_printf(DEBUG_NORMAL, "Blah : %d >= %d\n", ((framesize)-sizeof(*keydata)), keylen);
   if ((keylen != 0) && ((framesize)-sizeof(*keydata) >= keylen))
     {
       newkey = (u_char *)malloc(sizeof(u_char) * keylen);
@@ -278,27 +294,61 @@ int eapol_key_type1_decrypt(struct interface_data *thisint, char *inframe,
       debug_printf(DEBUG_INT, "Key after decryption : ");
       debug_hex_printf(DEBUG_INT, newkey, keylen);
 
-      if (set_wireless_key(thisint, newkey, keylen, keydata->key_index) != 0)
+      if (cardif_set_wep_key(thisint, newkey, keylen, keydata->key_index)
+	  != 0)
 	{
 	  rc = FALSE;
 	} else {
+
+	  //  If the unicast flag is set, start the warning timer.
+	  if (keydata->key_index & 0x80)
+	    {
+	      eapol_key_type1_set_rekey_prob_timer();
+	      eapol_key_type1_set_stale_key_warn_timer();
+	    }
+
 	  rc = TRUE;
+	  UNSET_FLAG(thisint->flags, ROAMED);
 	}
 
       free(newkey);
       newkey = NULL;
     } else {
       debug_printf(DEBUG_INT, "Using peer key!\n");
-      memcpy(&length, keydata->key_length, 2);
-      length = ntohs(length);
-      if (set_wireless_key(thisint, thisint->keyingMaterial, length, keydata->key_index) != 0)
+
+      if (config_get_display_friendly_warnings())
+	{
+	  debug_printf(DEBUG_NORMAL, "*WARNING* This AP uses the key "
+		       "generated during the authentication\nprocess.  If "
+		       "reauthentication doesn't happen frequently enough "
+		       "your connection\nmay not be very secure!\n");
+	}
+
+      if (cardif_set_wep_key(thisint, thisint->keyingMaterial, keylen, 
+			     keydata->key_index) != 0)
 	{
 	  rc = FALSE;
 	} else {
 	  rc = TRUE;
+
+	  //  If the unicast flag is set, start the warning timer.
+	  if (keydata->key_index & 0x80)
+	    {
+	      eapol_key_type1_set_rekey_prob_timer();
+	      eapol_key_type1_set_stale_key_warn_timer();
+	    }
 	}
     }
 
+  // If we reach this point, then we should remember the length of the
+  // keys for later comparison.
+  if (keydata->key_index & UNICAST_KEY)
+    {
+      thisint->statemachine->unicastKeyLen = keylen;
+    } else {
+      thisint->statemachine->broadcastKeyLen = keylen;
+    }
+
   return rc;
 }
 
@@ -309,19 +359,23 @@ int eapol_key_type1_decrypt(struct interface_data *thisint, char *inframe,
  * correct key, and make the call to set it.
  *
  **********************************/
-void eapol_key_type1_process(struct interface_data *thisint, char *inframe,
-			     int framesize)
+void eapol_key_type1_process(struct interface_data *thisint)
 {
   struct key_packet *keydata;
   struct eapol_header *eapolheader;
+  u_char *inframe;
+  int framesize;
   int framelen;
 
-  if ((!thisint) || (!inframe))
+  if (!thisint)
     {
       debug_printf(DEBUG_NORMAL, "Invalid data passed in to eapol_key_type1_process()!\n");
       return;
     }
 
+  inframe = thisint->recvframe;
+  framesize = thisint->recv_size;
+
   eapolheader = (struct eapol_header *)&inframe[OFFSET_PAST_MAC];
 
   framelen = ntohs(eapolheader->eapol_length);
@@ -346,4 +400,19 @@ void eapol_key_type1_process(struct interface_data *thisint, char *inframe,
       debug_printf(DEBUG_NORMAL, "Failed to set wireless key!\n");
       return;
     }
+
+  if ((thisint->statemachine->unicastKeyLen != 0) && 
+      (thisint->statemachine->broadcastKeyLen != 0) &&
+      (thisint->statemachine->unicastKeyLen != 
+       thisint->statemachine->broadcastKeyLen))
+    {
+
+      if (config_get_display_friendly_warnings())
+	{
+	  debug_printf(DEBUG_NORMAL, "[WARNING] Unicast and broadcast keys "
+		       "are different lengths!  Some cards/drivers/APs do "
+		       "not like this combination!\n");
+	}
+    }
+  thisint->recv_size = 0;
 }
