@@ -1,78 +1,17 @@
-/**
- * A client-side 802.1x implementation supporting EAP/TLS
- *
- * This code is released under both the GPL version 2 and BSD licenses.
- * Either license may be used.  The respective licenses are found below.
- *
- * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
- * All Rights Reserved
- *
- * --- GPL Version 2 License ---
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * --- BSD License ---
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *  - All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *       This product includes software developed by the University of
- *       Maryland at College Park and its contributors.
- *  - Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 /*******************************************************************
- * The driver function for a Linux application layer EAPOL 
- * implementation
+ * Routines for displaying/logging debug information.
+ *
+ * Licensed under the dual GPL/BSD license.  (See LICENSE file for more info.)
+ *
  * File: xsup_debug.c
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: xsup_debug.c,v 1.14 2004/08/11 04:38:12 chessing Exp $
- * $Date: 2004/08/11 04:38:12 $
+ * $Id: xsup_debug.c,v 1.19 2005/08/09 01:39:14 chessing Exp $
+ * $Date: 2005/08/09 01:39:14 $
  * $Log: xsup_debug.c,v $
- * Revision 1.14  2004/08/11 04:38:12  chessing
- *
- * Fixes to the new debug code.  (DEBUG_NORMAL stuff wasn't be displayed when it should have.)
- *
- * Revision 1.13  2004/08/11 04:01:17  chessing
- *
- * The -d option now supports letter designations for each debug level, in addition to 0..7.  (However, both letters and numbers can't be used at the same time!)  Also changed some formatting for the SNMP scoreboard.
- *
- * Revision 1.12  2004/06/15 03:22:29  chessing
- *
- * XSupplicant Release 1.0
+ * Revision 1.19  2005/08/09 01:39:14  chessing
+ * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
  *
  *******************************************************************/
@@ -81,15 +20,64 @@
 #include <stdarg.h>
 #include <strings.h>
 #include <string.h>
+#include <stdlib.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <netinet/in.h>
+#include <syslog.h>
 #include "xsup_debug.h"
 #include "config.h"
 
 unsigned char debug_level = 0;     // By default just show the "normal" stuff.
 int isdaemon = 0;
+int syslogging = 0;
 FILE *logfile = NULL;
 
+/***************************************************
+ *
+ * Convert a string to be all lowercase.
+ *
+ ***************************************************/
+void lowercase(char *instr)
+{
+  int i;
+
+  for (i=0;i<strlen(instr);i++)
+    {
+      instr[i] = tolower(instr[i]);
+    }
+}
+
+/***************************************************
+ *
+ * Return the logging facility the user asked us to use.
+ *
+ ***************************************************/
+int xsup_debug_get_facility(char *facility_str)
+{
+  int facility_num = LOG_DAEMON;
+
+  if (strcmp("cron", facility_str) == 0) facility_num = LOG_CRON;
+  if (strcmp("daemon", facility_str) == 0) facility_num = LOG_DAEMON;
+  if (strcmp("ftp", facility_str) == 0) facility_num = LOG_FTP;
+  if (strcmp("kern", facility_str) == 0) facility_num = LOG_KERN;
+  if (strcmp("local0", facility_str) == 0) facility_num = LOG_LOCAL0;
+  if (strcmp("local1", facility_str) == 0) facility_num = LOG_LOCAL1;
+  if (strcmp("local2", facility_str) == 0) facility_num = LOG_LOCAL2;
+  if (strcmp("local3", facility_str) == 0) facility_num = LOG_LOCAL3;
+  if (strcmp("local4", facility_str) == 0) facility_num = LOG_LOCAL4;
+  if (strcmp("local5", facility_str) == 0) facility_num = LOG_LOCAL5;
+  if (strcmp("local6", facility_str) == 0) facility_num = LOG_LOCAL6;
+  if (strcmp("local7", facility_str) == 0) facility_num = LOG_LOCAL7;
+  if (strcmp("lpr", facility_str) == 0) facility_num = LOG_LPR;
+  if (strcmp("mail", facility_str) == 0) facility_num = LOG_MAIL;
+  if (strcmp("news", facility_str) == 0) facility_num = LOG_NEWS;
+  if (strcmp("user", facility_str) == 0) facility_num = LOG_USER;
+  if (strcmp("uucp", facility_str) == 0) facility_num = LOG_UUCP;
+
+  return facility_num;
+}
+
 /***************************************************
  *
  * Remove an old logfile, and create a new one.
@@ -97,13 +85,39 @@ FILE *logfile = NULL;
  ***************************************************/
 void logfile_setup(char *logfilename)
 {
+  char *tempstr;
+  int facility;
+
   if (logfilename != NULL)
     {
       if (isdaemon != 2)
 	{
-	  unlink(logfilename);
+	  tempstr = (char *)malloc(strlen(logfilename)+1);
+	  if (tempstr == NULL)
+	    {
+	      printf("Couldn't allocate memory for temporary string! (%s:%d)\n",
+		     __FUNCTION__, __LINE__);
+	      return;
+	    }
 
-	  logfile = fopen(logfilename, "w+");
+	  strcpy(tempstr, logfilename);
+	  lowercase(tempstr);
+	  if (strcmp("syslog", tempstr) == 0)
+	    {
+	      free(tempstr);
+
+	      tempstr = config_get_log_facility();
+	      lowercase(tempstr);
+
+	      facility = xsup_debug_get_facility(tempstr);
+
+	      openlog("Xsupplicant", LOG_NDELAY, facility);
+	      syslogging = 1;
+	    } else {
+	      unlink(logfilename);
+
+	      logfile = fopen(logfilename, "w+");
+	    }
 	}
     }
 }
@@ -119,6 +133,10 @@ void logfile_cleanup()
     {
       fclose(logfile);
     }
+  if (syslogging == 1)
+    {
+      closelog();
+    }
 }
 
 /*************************************************
@@ -195,9 +213,12 @@ void debug_alpha_set_flags(char *new_flags)
  *************************************************/
 void ufprintf(FILE *fh, char *instr)
 {
-  if ((isdaemon == 2) || (fh == NULL))
+  if (((isdaemon == 2) || (fh == NULL)) && (syslogging != 1))
     {
       printf("%s", instr);
+    } else if (syslogging ==1) {
+      // XXX Consider ways of using other log levels.
+      syslog(LOG_ALERT, "%s", instr);
     } else {
       fprintf(fh, "%s", instr);
       fflush(fh);
@@ -218,7 +239,7 @@ void debug_setdaemon(int xdaemon)
     {
       close(0);
       close(1);
-      close(2);
+      close(2); 
     }
 }
 
@@ -385,13 +406,7 @@ void debug_printf(unsigned char level, char *fmt, ...)
       
       strcat((char *)&dumpstr, (char *)&temp);
 
-      if ((isdaemon == 2) || (logfile == NULL))
-	{
-	  printf("%s", dumpstr);
-	} else {
-	  fprintf(logfile, "%s", dumpstr);
-	  fflush(logfile);
-	}      
+      ufprintf(logfile, dumpstr);
 
       va_end(ap);
     }
@@ -405,18 +420,16 @@ void debug_printf(unsigned char level, char *fmt, ...)
  *************************************************/
 void debug_printf_nl(unsigned char level, char *fmt, ...)
 {
+  char temp[2048];
+
   if ((((level & debug_level) != 0x00) || (level == 0)) && (fmt != NULL))
     {
       va_list ap;
       va_start(ap, fmt);
 
-      if ((isdaemon == 2) || (logfile == NULL))
-	{ 
-	  vprintf(fmt, ap);
-	} else {
-	  vfprintf(logfile, fmt, ap);
-	  fflush(logfile);
-	}
+      vsnprintf((char *)&temp, 2048, fmt, ap);
+
+      ufprintf(logfile, temp);
 	  
       va_end(ap);
     }
